"use strict";
const ServerConnection_1 = require("./ServerConnection");
/**
 * Layer 3 implementation.
 * Created by emran on 9/3/16.
 */
class ApiConnection {
    /**
     * @param botToken The token of the bot.
     */
    constructor(botToken) {
        /**
         * @type {Array} Each item: the sent request id and its corresponding promise resolve function.
         */
        this.responseResolves = [];
        this.serverConnection = new ServerConnection_1.ServerConnection(botToken);
        this.serverConnection.setOnResponse((response) => {
            //foreach on responseResolves.
            for (let i = 0; i < this.responseResolves.length; i++) {
                let resolveIdPair = this.responseResolves[i];
                if (resolveIdPair.id == response.id) {
                    //The gotten response is for this resolve pack. so call the corresponding resolve.
                    resolveIdPair.resolveFunc(response);
                    this.responseResolves.splice(i, 1);
                    break;
                }
            }
        });
    }
    /**
     * Sends a request to the server and returns its server response within the returned promise.
     * @param request The request object to send.
     * @return {Promise<Response>} The promise that resolves when the response of the sent request is received.
     * @example
     * send(new SendMessageRequest(...)).then((resp: ResponsePacket)=> {
     *      //use resp
     * });
     */
    send(request) {
        return new Promise((resolve, reject) => {
            this.responseResolves.push(new ResolvePack(request.id, resolve)); //request id should be unique
            this.serverConnection.send(request);
        });
    }
    /**
     * Sets a callback to receive all {@link Message Message}s.
     * @param callback
     */
    setOnMessage(callback) {
        this.serverConnection.setOnMessage((receivedMessage) => {
            //ToDo handle read and received update
            if (receivedMessage.message)
                callback.call(this, receivedMessage.message, receivedMessage.peer);
        });
    }
}
exports.ApiConnection = ApiConnection;
/**
 * Resolve-Id pair
 */
class ResolvePack {
    /**
     * Instantiates a new Resolve-Id pair.
     * @param id The request id to listen for its corresponding response.
     * @param resolveFunc The resolve function of the corresponding Promise.
     */
    constructor(id, resolveFunc) {
        this.id = id;
        this.resolveFunc = resolveFunc;
    }
}
//# sourceMappingURL=ApiConnection.js.map