"use strict";
/**
 * Created by emran on 12/20/16.
 */
const WebSocket = require('ws');
class ReconnectingWebSocket {
    constructor(url, protocols) {
        protocols = protocols || [];
        // These can be altered by calling code.
        this.debug = false;
        this.reconnectInterval = 1000;
        this.timeoutInterval = 2000;
        this.forcedClose = false;
        this.timedOut = false;
        this.url = url;
        this.protocols = protocols;
        this.readyState = WebSocket.CONNECTING;
        this.onopen = function (event) {
        };
        this.onclose = function (event) {
        };
        this.onconnecting = function () {
        };
        this.onmessage = function (event) {
        };
        this.onerror = function (event) {
        };
        this.connect(url);
    }
    connect(reconnectAttempt) {
        this.ws = new WebSocket(this.url, this.protocols);
        this.onconnecting();
        if (this.debug || ReconnectingWebSocket.debugAll) {
            console.debug('ReconnectingWebSocket', 'attempt-connect', this.url);
        }
        let This = this;
        var timeout = setTimeout(function () {
            if (This.debug || ReconnectingWebSocket.debugAll) {
                console.debug('ReconnectingWebSocket', 'connection-timeout', This.url);
            }
            This.timedOut = true;
            This.ws.close();
            This.timedOut = false;
        }, this.timeoutInterval);
        this.ws.onopen = function (event) {
            clearTimeout(timeout);
            if (This.debug || ReconnectingWebSocket.debugAll) {
                console.debug('ReconnectingWebSocket', 'onopen', This.url);
            }
            This.readyState = WebSocket.OPEN;
            reconnectAttempt = false;
            This.onopen(event);
        };
        this.ws.onclose = function (event) {
            clearTimeout(timeout);
            This.ws = null;
            if (This.forcedClose) {
                This.readyState = WebSocket.CLOSED;
                This.onclose(event);
            }
            else {
                This.readyState = WebSocket.CONNECTING;
                This.onconnecting();
                if (!reconnectAttempt && !This.timedOut) {
                    if (This.debug || ReconnectingWebSocket.debugAll) {
                        console.debug('ReconnectingWebSocket', 'onclose', This.url);
                    }
                    This.onclose(event);
                }
                setTimeout(function () {
                    This.connect(true);
                }, This.reconnectInterval);
            }
        };
        this.ws.onmessage = function (event) {
            if (This.debug || ReconnectingWebSocket.debugAll) {
                console.debug('ReconnectingWebSocket', 'onmessage', This.url, event.data);
            }
            if (event.data)
                This.onmessage(event.data);
        };
        this.ws.onerror = function (event) {
            if (This.debug || ReconnectingWebSocket.debugAll) {
                console.debug('ReconnectingWebSocket', 'onerror', This.url, event);
            }
            This.onerror(event);
        };
        // For exception avoidance
        this.ws.on('error', (err) => {
        });
    }
    send(data) {
        if (this.ws) {
            if (this.debug || ReconnectingWebSocket.debugAll) {
                console.debug('ReconnectingWebSocket', 'send', this.url, data);
            }
            try {
                this.ws.send(data);
            }
            catch (e) {
                throw 'SOCKET_SEND_EXCEPTION error: ' + JSON.stringify(e);
            }
            return;
        }
        else {
            throw 'INVALID_STATE_ERR : Pausing to reconnect websocket';
        }
    }
    ;
    close() {
        if (this.ws) {
            this.forcedClose = true;
            this.ws.close();
        }
    }
    ;
    /**
     * Additional public API method to refresh the connection if still open (close, re-open).
     * For example, if the app suspects bad data / missed heart beats, it can try to refresh.
     */
    refresh() {
        if (this.ws) {
            this.ws.close();
        }
    }
    ;
}
ReconnectingWebSocket.debugAll = false;
exports.ReconnectingWebSocket = ReconnectingWebSocket;
//# sourceMappingURL=ReconnectingWebSocket.js.map