"use strict";
const SocketConnection_1 = require("./SocketConnection");
const ServerPacket_1 = require("../../models/serverMessages/ServerPacket");
const ResponseModelFactory_1 = require("../../models/factories/ResponseModelFactory");
const ReceivedMessage_1 = require("../../models/serverMessages/ReceivedMessage");
/**
 * Layer 2 implementation.
 *
 * Created by emran on 9/3/16.
 */
class ServerConnection {
    /**
     *
     * @param botToken The token of the bot.
     */
    constructor(botToken) {
        this.connection = new SocketConnection_1.SocketConnection(botToken, (packet) => {
            if (ServerPacket_1.ServerPacket.isResponse(packet)) {
                this.onResponseCB(ResponseModelFactory_1.ResponseModelFactory.build(packet));
            }
            else {
                this.onMessageCB(new ReceivedMessage_1.ReceivedMessage(packet));
            }
        });
        this.queue = new SendQueue(this.connection);
        // this.queue.start();
    }
    /**
     * Sends a request to the server as an object.
     * @param request The request object to send to the server.
     */
    send(request) {
        // this.connection.send(request.toJson());
        this.queue.push(request);
        this.queue.start();
    }
    /**
     *
     * @param callback A callback that gets called when a message is received from the end-user. The callback should be like this: function(message: ReceivedMessage).
     */
    setOnMessage(callback) {
        this.onMessageCB = callback;
    }
    /**
     *
     * @param callback A callback that gets called when a response is received from the server. The callback should be like this: function(response: ResponsePacket)
     */
    setOnResponse(callback) {
        this.onResponseCB = callback;
    }
}
exports.ServerConnection = ServerConnection;
class SendQueue {
    constructor(connection) {
        this.isStarted = false;
        this.nextTryTimer = null;
        this.requests = [];
        this.connection = connection;
    }
    /**
     * Starts checking the queued request and send if connection is ok.
     * @throws Exception on trying to start an already started queue.
     */
    start() {
        if (this.isStarted)
            // throw "SendQueue is already started.";
            return;
        this.isStarted = true;
        let This = this;
        let runner = function () {
            if (This.requests.length > 0) {
                let req = This.requests.shift();
                while (req) {
                    (function (req) {
                        setTimeout(() => {
                            This.connection.send(req.toJson()).catch((e) => {
                                This.requests.push(req);
                                This.start();
                            });
                        }, 0);
                    })(req);
                    req = This.requests.shift();
                }
            }
            if (This.requests.length > 0)
                This.nextTryTimer = setTimeout(runner, 0);
            else
                This.isStarted = false;
        };
        setTimeout(runner, 0); // this timeout is set to start the runner
    }
    /**
     * Stops the queue checking process. Does nothing if the process is already stopped.
     */
    stop() {
        if (this.nextTryTimer != null)
            clearTimeout(this.nextTryTimer);
        this.isStarted = false;
    }
    /**
     * Pushes the given request to the send queue.
     * @param request
     */
    push(request) {
        this.requests.push(request);
    }
}
exports.SendQueue = SendQueue;
//# sourceMappingURL=ServerConnection.js.map