/**
 * Created by amin on 5/3/16.
 * Modified by emran on 10/19/16.
 */
"use strict";
const ApiConst_1 = require("../../api/ApiConst");
const Logger_1 = require("../../utils/Logger");
const ReconnectingWebSocket_1 = require("./ReconnectingWebSocket");
class SocketConnection {
    /**
     * construct class with botToken
     * @param botToken
     * @param callback
     */
    constructor(botToken, callback) {
        this._isOnline = false;
        this.botToken = botToken;
        this.onReceiveCallBack = callback;
        this.connect();
    }
    /**
     * connect to server and set callback for handling Errors
     * @returns {Promise<any>}
     * */
    connect() {
        this.socketClient = new ReconnectingWebSocket_1.ReconnectingWebSocket(ApiConst_1.ApiConst.BASE_API_URL + this.botToken, []);
        this.socketClient.reconnectInterval = 5000;
        this.socketClient.onopen = (e) => {
            Logger_1.Logger.log('Socket Connected!');
        };
        this.socketClient.onmessage = this.onReceiveCallBack;
        this.socketClient.onconnecting = () => {
            Logger_1.Logger.log('Connecting...');
        };
        this.socketClient.onerror = (e) => {
            Logger_1.Logger.log('Error: ' + e);
        };
    }
    /**
     *
     * @param jsonString string of message json
     * if connection is stable send message and return true else return false
     * @returns {Promise<boolean>}
     */
    send(jsonString) {
        return new Promise((resolve, reject) => {
            try {
                this.socketClient.send(jsonString);
                resolve(true);
            }
            catch (e) {
                reject();
            }
        });
    }
    /**
     * set callback to on message receive from socket
     * @param callback on message recived callback
     * @return {void}
     */
    setOnReceive(callback) {
        this.onReceiveCallBack = callback;
    }
}
exports.SocketConnection = SocketConnection;
//# sourceMappingURL=SocketConnection.js.map