"use strict";
const ServerPacket_1 = require("../serverMessages/ServerPacket");
const SendMessageResponse_1 = require("../serverMessages/SendMessageResponse");
const UnknownResponse_1 = require("../serverMessages/UnknownResponse");
const GetKeysResponse_1 = require("../serverMessages/GetKeysResponse");
/**
 *
 * Created by aliakbarbadri on 9/5/16.
 */
class ResponseModelFactory {
    /**
     * Builds the corresponding ServerPacket class from the given json.
     * @param jsonPacket A json string that represents a response.
     * @return {Response} The corresponding ServerPacket class. Returns null if is not a ServerPacket or can't convert to any predefined Server class.
     */
    static build(jsonPacket) {
        let objPacket = JSON.parse(jsonPacket);
        if (typeof objPacket === 'undefined') {
            return null;
        }
        if (ServerPacket_1.ServerPacket.isResponse(jsonPacket)) {
            let body = objPacket.body;
            if (body.date != null)
                return new SendMessageResponse_1.SendMessageResponse(jsonPacket);
            else if (body.value != null)
                return new GetKeysResponse_1.GetKeysResponse(jsonPacket);
            else
                return new UnknownResponse_1.UnknownResponse(jsonPacket);
        }
        return null;
    }
}
exports.ResponseModelFactory = ResponseModelFactory;
//# sourceMappingURL=ResponseModelFactory.js.map