"use strict";
const TextMessage_1 = require("../../models/message/TextMessage");
/**
 * A class that is sensitive to some regex or strings.
 * Created by amin on 9/3/16.
 * Modified by emran on 9/15/16
 */
class TextMessageSensitive {
    constructor(keywords) {
        this.keywords = [];
        if (Array.isArray(keywords)) {
            for (let str of keywords) {
                this.keywords.push(this.toRegExp(str));
            }
        }
        else if (typeof keywords === 'string') {
            this.keywords.push(this.toRegExp(keywords));
        }
        else if (keywords instanceof RegExp) {
            this.keywords.push(keywords);
        }
    }
    /**
     * Creates a simple RegExp object from a string, that matches with exactly that string.
     *
     * @param str The string to make to the type of RegExp.
     * @return {RegExp}
     */
    toRegExp(str) {
        return new RegExp(str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&"));
    }
    /**
     * Checks if the given text matches any subscribed keyword
     * @param text
     * @return {boolean}
     */
    match(message) {
        if (message instanceof TextMessage_1.TextMessage) {
            let txtMsg = message;
            for (let rex of this.keywords) {
                if (rex.test(txtMsg.text))
                    return true;
            }
        }
        return false;
    }
}
exports.TextMessageSensitive = TextMessageSensitive;
//# sourceMappingURL=TextMessageSensitive.js.map